// Coincides with 10.41
package main

import (
	migration "bitbucket.org/prmunty/util/dbmigration/db"
	"bitbucket.org/prmunty/util/ethernet"
	"database/sql"
	_ "github.com/mattn/go-sqlite3"
	"log"
	"net"
)

func main() {

	log.Println("starting...")

	const version float64 = 6.0

	migration.Migrate(version, func(tx *sql.Tx) (err error) {
		return migration.DoUntilErr(tx,
			UpdateEquipmentTable,
			addRefrigerantDetection,
			CreateModeOrdinalsTable,
			InsertModeOrdinals,
			UpdateFreezeTable,
			InsertFreezeData,
			createUnderFloorSensorTable,
			addUnderFloorSensors,
			RecreateIndexes,
		)
	})
}

func UpdateEquipmentTable(tx *sql.Tx) error {
	rows, err := tx.Query(
		`SELECT ip FROM equipment`,
	);

	if err != nil {
		return err
	}

	for rows.Next() {
		var ip string

		err = rows.Scan(&ip)
		if err != nil {
			return err
		}

		netIp := net.ParseIP(ip)

		if ethernet.IsLocalIP(netIp) {
			if _, err = tx.Exec(
				`DELETE FROM equipment WHERE ip = ?`,
				ip,
			); err != nil {
				return err
			}
		}
	}

	return nil
}

func createUnderFloorSensorTable(tx *sql.Tx) error{

	if _, err := tx.Exec(
		`DROP TABLE IF EXISTS global_settings_underfloor_names; CREATE TABLE global_settings_underfloor_names(input_letter Text, name Text)`,
	); err != nil {
		return err
	}
	return nil

}



func addUnderFloorSensors(tx *sql.Tx) (err error) {


	keys := [12] string {
		"A",
		"B",
		"C",
		"D",
		"E",
		"F",
		"G",
		"H",
		"I",
		"J",
		"K",
		"L",
	}
	sensors := [12]string {
		"Underfloor Temp. A",
		"Underfloor Temp. B",
		"Underfloor Temp. C",
		"Underfloor Temp. D",
		"Underfloor Temp. E",
		"Underfloor Temp. F",
		"Underfloor Temp. G",
		"Underfloor Temp. H",
		"Underfloor Temp. I",
		"Underfloor Temp. J",
		"Underfloor Temp. K",
		"Underfloor Temp. L",
	}

	for index, sensorName := range sensors {
		_, err := tx.Exec(`INSERT INTO "global_settings_underfloor_names" ("input_letter", "name") VALUES (?,?);`, keys[index], sensorName)

		if err != nil {
			return err
		}
	}

	return

}




func addRefrigerantDetection(tx *sql.Tx) (err error) {
	detectors := [4]string{
		"Refrigerant Detector E",
		"Refrigerant Detector F",
		"Refrigerant Detector G",
		"Refrigerant Detector H",
	}

	for _, de := range detectors {
		_, err := tx.Exec(`INSERT INTO global_settings_refrigerant_detectors_engine (name) VALUES(?);`, de)
		if err != nil {
			return err
		}
	}

	return
}

func CreateModeOrdinalsTable(tx *sql.Tx) error {
	if _, err := tx.Exec(
		`DROP TABLE IF EXISTS mode_ordinals; CREATE TABLE mode_ordinals(mode_data Integer,text_index Integer,ordinal Integer)`,
	); err != nil {
		return err
	}

	return nil
}

func InsertModeOrdinals(tx *sql.Tx) error {
	userdefinedIds := map[int]int{
		1:1,
		2:2,
		3:3,
		4:4,
		5:5,
		6:6,
		7:11,
		8:12,
		9:15,
		10:16,
		11:17,
		12:18,
		13:19,
		14:20,
		15:21,
		16:22,
		17:23,
		18:24,
		19:25,
		20:26,
		21:13,
		22:14,
		33:7,
		34:8,
		35:9,
		36:10,
		38:27,
		39:28,
		40:47,
		41:48,
		42:49,
		43:50,
		44:51,
		45:52,
		46:53,
		47:54,
		48:55,
		49:56,
		51:29,
		52:30,
		53:31,
		54:32,
		55:33,
		56:34,
		57:35,
		58:36,
		59:37,
		60:38,
		61:39,
		62:40,
		63:41,
		64:42,
		65:43,
		66:44,
		67:45,
		68:46,
	}

	for key, value := range userdefinedIds {
		if _, err := tx.Exec(
			`INSERT INTO mode_ordinals(mode_data, text_index, ordinal) VALUES( ?, ?, ? )`,
			496,
			key,
			value,
		); err != nil {
			return err
		}
	}

	for key, value := range userdefinedIds {
		if _, err := tx.Exec(
			`INSERT INTO mode_ordinals(mode_data, text_index, ordinal) VALUES( ?, ?, ? )`,
			669,
			key,
			value,
		); err != nil {
			return err
		}
	}

	return nil
}

func UpdateFreezeTable(tx *sql.Tx) error {
	if _, err := tx.Exec(
		`DROP TABLE IF EXISTS "freeze_data_engine"; CREATE TABLE "freeze_data_engine" ("freeze_id" Integer PRIMARY KEY, "table_id" Integer, "selected" Boolean,"ordinal" Integer);`,
	); err != nil {
		return err
	}

	return nil
}


func InsertFreezeData(tx *sql.Tx) error {
	freezeIds := map[int]int {
		143:1,
		553:2,
		144:3,
		145:4,
		146:5,
		147:6,
		150:15,
		151:16,
		152:17,
		153:18,
		319:47,
		320:48,
		321:49,
		322:50,
		323:51,
		324:52,
		325:53,
		326:54,
		327:55,
		328:56,
		2917:29,
		2918:30,
		2919:31,
		2920:32,
		2921:33,
		2922:34,
		2923:35,
		2924:36,
		2925:37,
		2926:38,
		2927:39,
		2928:40,
		2929:41,
		2930:42,
		2931:43,
		2932:44,
		2933:45,
		2934:46,
		4363:19,
		4364:20,
		4365:21,
		4366:22,
		4367:23,
		4368:24,
		4369:25,
		4370:26,
		4373:11,
		4374:12,
		4375:13,
		4376:14,
		4418:27,
		4419:28,
		4448:7,
		4449:8,
		4450:9,
		4451:10,
		180:57,
		181:58,
		182:59,
		183:60,
		2534:61,
		2535:62,
		2536:63,
		2537:64,
  }

	for key, value := range freezeIds {
		table_id := 0

		if _, err := tx.Exec(
			"INSERT INTO \"freeze_data_engine\" (\"freeze_id\",\"table_id\",\"selected\",\"ordinal\") VALUES(?, ?, ?, ?);",
			key,
			table_id,
			1,
			value,
		); err != nil {
			return err
		}
	}

	return nil
}

func RecreateIndexes(tx *sql.Tx) error {
	drops := [6]string{
		"DROP INDEX IF EXISTS [idx_data_key]",
		"DROP INDEX IF EXISTS \"data_keyIdx\"",
		"DROP INDEX IF EXISTS \"drop_downsIdx\"",
		"DROP INDEX IF EXISTS \"index_authid\"",
		"DROP INDEX IF EXISTS \"languagesIdx\"",
		"DROP INDEX IF EXISTS \"idx_languages\"",
	}

	for _, value := range drops {
		if _, err := tx.Exec(
			value,
		); err != nil {
			return err
		}
	}

	adds := [6]string{
		"CREATE INDEX [idx_data_key] ON [data_key] ([id] ASC)",
		"CREATE INDEX \"data_keyIdx\" ON \"data_key\"( \"language_id\", \"type\" )",
		"CREATE INDEX \"drop_downsIdx\" ON \"drop_downs\"( \"mode_data\", \"language_id\" )",
		"CREATE INDEX \"index_authid\" ON \"global_settings_smtp_auth\"( \"authid\" )",
		"CREATE INDEX \"languagesIdx\" ON \"languages\"( \"name\" )",
		"CREATE INDEX \"idx_languages\" ON \"languages\"( \"language_id\" )",
	}

	for _, value := range adds {
		if _, err := tx.Exec(
			value,
		); err != nil {
			return err
		}
	}

	return nil
}
